import logging
from pathlib import Path
from typing import List, Optional
import pytest
from datalad_installer import (
    ComponentRequest,
    DataladInstaller,
    HelpRequest,
    Option,
    ParsedArgs,
    SudoConfirm,
    UsageError,
    VersionRequest,
)


@pytest.mark.parametrize(
    "args,parsed",
    [
        ([], ParsedArgs({}, [])),
        (["datalad"], ParsedArgs({}, [ComponentRequest(name="datalad")])),
        (
            ["--log-level", "INFO", "datalad"],
            ParsedArgs(
                {"log_level": logging.INFO},
                [ComponentRequest(name="datalad")],
            ),
        ),
        (
            ["--log-level", "info", "datalad"],
            ParsedArgs(
                {"log_level": logging.INFO},
                [ComponentRequest(name="datalad")],
            ),
        ),
        (
            ["--log-level", "15", "datalad"],
            ParsedArgs(
                {"log_level": 15},
                [ComponentRequest(name="datalad")],
            ),
        ),
        (
            ["--sudo", "ask", "datalad"],
            ParsedArgs(
                {"sudo": SudoConfirm.ASK},
                [ComponentRequest(name="datalad")],
            ),
        ),
        (
            ["--sudo", "error", "datalad"],
            ParsedArgs(
                {"sudo": SudoConfirm.ERROR},
                [ComponentRequest(name="datalad")],
            ),
        ),
        (
            ["--sudo", "ok", "datalad"],
            ParsedArgs(
                {"sudo": SudoConfirm.OK},
                [ComponentRequest(name="datalad")],
            ),
        ),
        (
            ["-E", "/path/to/file", "datalad"],
            ParsedArgs(
                {"env_write_file": [Path("/path/to/file")]},
                [ComponentRequest(name="datalad")],
            ),
        ),
        (
            ["-E", "/path/to/file", "--env-write-file=writefile", "datalad"],
            ParsedArgs(
                {"env_write_file": [Path("/path/to/file"), Path("writefile")]},
                [ComponentRequest(name="datalad")],
            ),
        ),
        (["--help"], HelpRequest(None)),
        (["--help", "datalad"], HelpRequest(None)),
        (["--help", "datalad", "--invalid"], HelpRequest(None)),
        (["datalad", "--help"], HelpRequest("datalad")),
        (["datalad", "--help", "invalid"], HelpRequest("datalad")),
        (["datalad", "--help", "git-annex", "--invalid"], HelpRequest("datalad")),
        (["--version"], VersionRequest()),
        (["--version", "datalad"], VersionRequest()),
        (["--version", "datalad", "--invalid"], VersionRequest()),
        (["--version", "invalid"], VersionRequest()),
        (
            ["git-annex", "-e", "--extra-opt"],
            ParsedArgs(
                {},
                [
                    ComponentRequest(
                        name="git-annex",
                        extra_args=["--extra-opt"],
                    )
                ],
            ),
        ),
        (
            ["git-annex", "-e", "--extra --opt"],
            ParsedArgs(
                {},
                [
                    ComponentRequest(
                        name="git-annex",
                        extra_args=["--extra", "--opt"],
                    )
                ],
            ),
        ),
        (
            [
                "git-annex",
                "-e",
                "--extra --opt",
                "datalad",
                "--extra-args",
                "--extra=opt",
            ],
            ParsedArgs(
                {},
                [
                    ComponentRequest(
                        name="git-annex",
                        extra_args=["--extra", "--opt"],
                    ),
                    ComponentRequest(
                        name="datalad",
                        extra_args=["--extra=opt"],
                    ),
                ],
            ),
        ),
        (
            ["venv", "--path", "/path/to/venv", "datalad", "--extras", "all"],
            ParsedArgs(
                {},
                [
                    ComponentRequest(
                        name="venv",
                        path=Path("/path/to/venv"),
                    ),
                    ComponentRequest(name="datalad", extras="all"),
                ],
            ),
        ),
        (
            ["datalad=0.13.0"],
            ParsedArgs({}, [ComponentRequest(name="datalad", version="0.13.0")]),
        ),
        (
            ["datalad=0.13.0", "-e", "-a -b -c"],
            ParsedArgs(
                {},
                [
                    ComponentRequest(
                        name="datalad",
                        version="0.13.0",
                        extra_args=["-a", "-b", "-c"],
                    )
                ],
            ),
        ),
        (
            ["git-annex", "--build-dep"],
            ParsedArgs(
                {},
                [ComponentRequest(name="git-annex", build_dep=True)],
            ),
        ),
        (
            ["git-annex", "--method", "auto"],
            ParsedArgs(
                {},
                [ComponentRequest(name="git-annex", method="auto")],
            ),
        ),
        (
            ["git-annex", "--method", "apt"],
            ParsedArgs(
                {},
                [ComponentRequest(name="git-annex", method="apt")],
            ),
        ),
        (
            ["conda-env", "--name", "foo"],
            ParsedArgs(
                {},
                [ComponentRequest(name="conda-env", envname="foo")],
            ),
        ),
    ],
)
def test_parse_args(args: List[str], parsed: ParsedArgs) -> None:
    assert DataladInstaller.parse_args(args) == parsed


@pytest.mark.parametrize(
    "args,message,component",
    [
        (["--invalid"], "option --invalid not recognized", None),
        (["--log-level", "42", "--invalid"], "option --invalid not recognized", None),
        (["--log-level", "invalid"], "Invalid log level: 'invalid'", None),
        (["--log-level"], "option --log-level requires argument", None),
        (["datalad", "--invalid"], "option --invalid not recognized", "datalad"),
        (["datalad=", "--invalid"], "Version must be nonempty", "datalad"),
        (["=0.13.0", "--invalid"], "Component name must be nonempty", None),
        (["invalid"], "Unknown component: 'invalid'", None),
        (["venv=1.2.3"], "venv component does not take a version", "venv"),
        (
            ["git-annex", "--method", "pip"],
            "Invalid choice for --method option: 'pip'",
            "git-annex",
        ),
        (
            ["venv", "--extra-args", "--foo 'bar"],
            '"--foo \'bar": No closing quotation',
            "venv",
        ),
        (["--sudo", "invalid"], "Invalid choice for --sudo option: 'invalid'", None),
    ],
)
def test_parse_args_errors(
    args: List[str], message: str, component: Optional[str]
) -> None:
    with pytest.raises(UsageError) as excinfo:
        DataladInstaller.parse_args(args)
    assert str(excinfo.value) == message
    assert excinfo.value.component == component


@pytest.mark.parametrize(
    "option,helptext",
    [
        (
            Option("-f", "--foo"),
            "  -f, --foo FOO",
        ),
        (
            Option("-f", "--foo", is_flag=True),
            "  -f, --foo",
        ),
        (
            Option("-f", "--foo", help="Foo all the things"),
            "  -f, --foo FOO                   Foo all the things",
        ),
        (
            Option("-f", "--foo", is_flag=True, help="Foo all the things"),
            "  -f, --foo                       Foo all the things",
        ),
        (
            Option("-f", "--foo", metavar="PARAM", help="Foo all the things"),
            "  -f, --foo PARAM                 Foo all the things",
        ),
        (
            Option(
                "-f",
                "--foo",
                choices=["apple", "banana", "coconut"],
                help="Foo all the things",
            ),
            "  -f, --foo [apple|banana|coconut]\n"
            "                                  Foo all the things",
        ),
        (
            Option(
                "-f",
                "--foo",
                help=(
                    "Lorem ipsum dolor sit amet, consectetur adipisicing elit,"
                    " sed do eiusmod tempor incididunt ut labore et dolore"
                    " magna aliqua."
                ),
            ),
            "  -f, --foo FOO                   Lorem ipsum dolor sit amet, consectetur\n"
            "                                  adipisicing elit, sed do eiusmod tempor\n"
            "                                  incididunt ut labore et dolore magna\n"
            "                                  aliqua.",
        ),
    ],
)
def test_option_get_help(option: Option, helptext: str) -> None:
    assert option.get_help() == helptext


def test_global_short_help() -> None:
    assert DataladInstaller.short_help("datalad_installer") == (
        "Usage: datalad_installer [<options>] [COMPONENT[=VERSION] [<options>]] ..."
    )


def test_component_short_help() -> None:
    assert DataladInstaller.short_help("datalad_installer", "miniconda") == (
        "Usage: datalad_installer [<options>] miniconda [<options>]"
    )


def test_versioned_component_short_help() -> None:
    assert DataladInstaller.short_help("datalad_installer", "git-annex") == (
        "Usage: datalad_installer [<options>] git-annex[=VERSION] [<options>]"
    )


def test_global_long_help() -> None:
    assert DataladInstaller.long_help("datalad_installer") == (
        "Usage: datalad_installer [<options>] [COMPONENT[=VERSION] [<options>]] ...\n"
        "\n"
        "  Installation script for Datalad and related components\n"
        "\n"
        "Options:\n"
        "  -E, --env-write-file ENV_WRITE_FILE\n"
        "                                  Append PATH modifications and other\n"
        "                                  shell commands to the given file; can be\n"
        "                                  given multiple times\n"
        "  -l, --log-level LEVEL           Set logging level [default: INFO]\n"
        "  --sudo [ask|error|ok]           How to handle sudo commands [default:\n"
        "                                  ask]\n"
        "  -V, --version                   Show program version and exit\n"
        "  -h, --help                      Show this help information and exit\n"
        "\n"
        "Components:\n"
        "  conda-env                Create a Conda environment\n"
        "  datalad                  Install Datalad\n"
        "  git-annex                Install git-annex\n"
        "  git-annex-remote-rclone  Install git-annex-remote-rclone\n"
        "  miniconda                Install Miniconda\n"
        "  neurodebian              Install & configure NeuroDebian\n"
        "  rclone                   Install rclone\n"
        "  venv                     Create a Python virtual environment"
    )


def test_component_long_help() -> None:
    assert DataladInstaller.long_help("datalad_installer", "miniconda") == (
        "Usage: datalad_installer [<options>] miniconda [<options>]\n"
        "\n"
        "  Install Miniconda\n"
        "\n"
        "Options:\n"
        "  --batch                         Run in batch (noninteractive) mode\n"
        "  -e, --extra-args EXTRA_ARGS     Extra arguments to pass to the install\n"
        "                                  command\n"
        "  --path PATH                     Install Miniconda at the given path\n"
        "  --spec SPEC                     Space-separated list of package\n"
        "                                  specifiers to install in the Miniconda\n"
        "                                  environment\n"
        "  -h, --help                      Show this help information and exit"
    )


def test_versioned_component_long_help() -> None:
    assert DataladInstaller.long_help("datalad_installer", "git-annex") == (
        "Usage: datalad_installer [<options>] git-annex[=VERSION] [<options>]\n"
        "\n"
        "  Install git-annex\n"
        "\n"
        "Options:\n"
        "  --build-dep                     Install build-dep instead of the package\n"
        "  -e, --extra-args EXTRA_ARGS     Extra arguments to pass to the install\n"
        "                                  command\n"
        "  --install-dir DIR               Directory in which to unpack the `*.deb`\n"
        "  -m, --method [auto|apt|brew|neurodebian|deb-url|autobuild|snapshot|"
        "conda|datalad/git-annex:tested|datalad/git-annex|"
        "datalad/git-annex:release|datalad/packages|dmg]\n"
        "                                  Select the installation method to use\n"
        "  --path PATH                     Path to local `*.dmg` to install\n"
        "  --url URL                       URL from which to download `*.deb` file\n"
        "  -h, --help                      Show this help information and exit"
    )



import json
import logging
from pathlib import Path
import shlex
import shutil
import subprocess
import sys
import tempfile
import pytest
from pytest_mock import MockerFixture
import datalad_installer
from datalad_installer import (
    ON_LINUX,
    ON_MACOS,
    ON_POSIX,
    ON_WINDOWS,
    DataladGitAnnexReleaseBuildInstaller,
    main,
)


def bin_path(binname: str) -> Path:
    if ON_WINDOWS:
        return Path("Scripts", binname + ".exe")
    else:
        return Path("bin", binname)


@pytest.fixture(autouse=True)
def capture_all_logs(caplog: pytest.LogCaptureFixture) -> None:
    caplog.set_level(logging.DEBUG)


@pytest.mark.miniconda
def test_install_miniconda(tmp_path: Path) -> None:
    miniconda_path = tmp_path / "conda"
    r = main(
        [
            "datalad_installer.py",
            "miniconda",
            "--batch",
            "--path",
            str(miniconda_path),
        ]
    )
    assert r == 0
    assert (miniconda_path / bin_path("conda")).exists()
    assert (
        "conda activate test"
        in subprocess.run(
            [str(miniconda_path / bin_path("conda")), "create", "-n", "test", "-y"],
            stdout=subprocess.PIPE,
            universal_newlines=True,
            check=True,
        ).stdout
    )


@pytest.mark.miniconda
def test_install_miniconda_autogen_path(monkeypatch: pytest.MonkeyPatch) -> None:
    # Override TMPDIR with a path that will be cleaned up afterwards (We can't
    # use tmp_path here, as that's apparently always in the user temp folder on
    # Windows regardless of the external value of TMPDIR.)
    with tempfile.TemporaryDirectory() as newtmp:
        monkeypatch.setenv("TMPDIR", newtmp)
        tempfile.tempdir = None  # Reset cache
        r = main(
            [
                "datalad_installer.py",
                "miniconda",
                "--batch",
            ]
        )
        assert r == 0
        (miniconda_path,) = Path(newtmp).glob("dl-miniconda-*")
        assert (miniconda_path / bin_path("conda")).exists()
        assert (
            "conda activate test"
            in subprocess.run(
                [str(miniconda_path / bin_path("conda")), "create", "-n", "test", "-y"],
                stdout=subprocess.PIPE,
                universal_newlines=True,
                check=True,
            ).stdout
        )
    tempfile.tempdir = None  # Reset cache


@pytest.mark.miniconda
def test_install_env_write_file_miniconda_conda_env(tmp_path: Path) -> None:
    env_write_file = tmp_path / "env.sh"
    miniconda_path = tmp_path / "conda"
    r = main(
        [
            "datalad_installer.py",
            "-E",
            str(env_write_file),
            "miniconda",
            "--batch",
            "--path",
            str(miniconda_path),
            "conda-env",
            "-n",
            "foo",
        ]
    )
    assert r == 0
    assert (miniconda_path / bin_path("conda")).exists()
    assert (miniconda_path / "envs" / "foo").exists()
    ewf_path = str(env_write_file)
    if ON_WINDOWS:
        ewf_path = "/" + ewf_path.replace("\\", "/").replace(":", "", 1)
        bash = r"C:\Program Files\Git\bin\bash.EXE"
    else:
        bash = "bash"
    info = json.loads(
        subprocess.run(
            [
                bash,
                "-c",
                f"source {shlex.quote(ewf_path)} && conda info --json",
            ],
            stdout=subprocess.PIPE,
            universal_newlines=True,
            check=True,
        ).stdout
    )
    assert info["active_prefix_name"] == "foo"
    assert info["conda_prefix"] == str(miniconda_path)


@pytest.mark.miniconda
def test_install_miniconda_datalad(tmp_path: Path) -> None:
    miniconda_path = tmp_path / "conda"
    r = main(
        [
            "datalad_installer.py",
            "miniconda",
            "--batch",
            "--path",
            str(miniconda_path),
            "datalad",
        ]
    )
    assert r == 0
    assert (miniconda_path / bin_path("conda")).exists()
    assert (miniconda_path / bin_path("datalad")).exists()


@pytest.mark.miniconda
def test_install_miniconda_conda_env_datalad(tmp_path: Path) -> None:
    miniconda_path = tmp_path / "conda"
    r = main(
        [
            "datalad_installer.py",
            "miniconda",
            "--batch",
            "--path",
            str(miniconda_path),
            "conda-env",
            "-n",
            "foo",
            "datalad",
        ]
    )
    assert r == 0
    assert (miniconda_path / bin_path("conda")).exists()
    assert not (miniconda_path / bin_path("datalad")).exists()
    assert (miniconda_path / "envs" / "foo").exists()
    assert (miniconda_path / "envs" / "foo" / bin_path("datalad")).exists()


@pytest.mark.miniconda
def test_install_venv_miniconda_datalad(tmp_path: Path) -> None:
    venv_path = tmp_path / "venv"
    miniconda_path = tmp_path / "conda"
    r = main(
        [
            "datalad_installer.py",
            "venv",
            "--path",
            str(venv_path),
            "miniconda",
            "--batch",
            "--path",
            str(miniconda_path),
            "datalad",
        ]
    )
    assert r == 0
    assert (venv_path / bin_path("python")).exists()
    assert not (venv_path / bin_path("datalad")).exists()
    assert (miniconda_path / bin_path("conda")).exists()
    assert (miniconda_path / bin_path("datalad")).exists()


@pytest.mark.miniconda
def test_install_venv_miniconda_conda_env_datalad(tmp_path: Path) -> None:
    venv_path = tmp_path / "venv"
    miniconda_path = tmp_path / "conda"
    r = main(
        [
            "datalad_installer.py",
            "venv",
            "--path",
            str(venv_path),
            "miniconda",
            "--batch",
            "--path",
            str(miniconda_path),
            "conda-env",
            "-n",
            "foo",
            "datalad",
        ]
    )
    assert r == 0
    assert (venv_path / bin_path("python")).exists()
    assert not (venv_path / bin_path("datalad")).exists()
    assert (miniconda_path / bin_path("conda")).exists()
    assert not (miniconda_path / bin_path("datalad")).exists()
    assert (miniconda_path / "envs" / "foo").exists()
    assert (miniconda_path / "envs" / "foo" / bin_path("datalad")).exists()


def test_install_venv_datalad(tmp_path: Path) -> None:
    venv_path = tmp_path / "venv"
    r = main(
        [
            "datalad_installer.py",
            "venv",
            "--path",
            str(venv_path),
            "datalad",
        ]
    )
    assert r == 0
    assert (venv_path / bin_path("python")).exists()
    assert (venv_path / bin_path("datalad")).exists()


@pytest.mark.skipif(
    sys.version_info[:2] < (3, 7), reason="dev pip no longer supports Python < 3.7"
)
def test_install_venv_dev_pip_datalad(tmp_path: Path) -> None:
    venv_path = tmp_path / "venv"
    r = main(
        [
            "datalad_installer.py",
            "venv",
            "--path",
            str(venv_path),
            "--dev-pip",
            "datalad",
        ]
    )
    assert r == 0
    assert (venv_path / bin_path("python")).exists()
    assert (venv_path / bin_path("datalad")).exists()


@pytest.mark.miniconda
def test_install_miniconda_conda_env_venv_datalad(tmp_path: Path) -> None:
    venv_path = tmp_path / "venv"
    miniconda_path = tmp_path / "conda"
    r = main(
        [
            "datalad_installer.py",
            "miniconda",
            "--batch",
            "--path",
            str(miniconda_path),
            "conda-env",
            "-n",
            "foo",
            "venv",
            "--path",
            str(venv_path),
            "datalad",
        ]
    )
    assert r == 0
    assert (venv_path / bin_path("python")).exists()
    assert (venv_path / bin_path("datalad")).exists()
    assert (miniconda_path / bin_path("conda")).exists()
    assert not (miniconda_path / bin_path("datalad")).exists()
    assert (miniconda_path / "envs" / "foo").exists()
    assert not (miniconda_path / "envs" / "foo" / bin_path("datalad")).exists()


@pytest.mark.ci_only
@pytest.mark.skipif(
    not ON_LINUX or shutil.which("apt-get") is None,
    reason="requires Debian-based system",
)
def test_install_neurodebian_sudo_ok(mocker: MockerFixture) -> None:
    spy = mocker.spy(datalad_installer, "runcmd")
    r = main(["datalad_installer.py", "--sudo=ok", "neurodebian"])
    assert r == 0
    assert spy.call_args_list[-2] == mocker.call(
        "sudo", "apt-get", "install", "-qy", "neurodebian", env=mocker.ANY
    )
    assert spy.call_args_list[-2][1]["env"]["DEBIAN_FRONTEND"] == "noninteractive"
    assert spy.call_args_list[-1] == mocker.call("nd-configurerepo")
    assert (
        subprocess.run(
            [
                "dpkg-query",
                "-Wf",
                "${db:Status-Abbrev}",
                "neurodebian",
            ],
            stdout=subprocess.PIPE,
            universal_newlines=True,
            check=True,
        ).stdout
        == "ii "
    )


@pytest.mark.ci_only
@pytest.mark.skipif(
    not ON_MACOS or shutil.which("brew") is None, reason="requires macOS with Homebrew"
)
def test_install_git_annex_brew(mocker: MockerFixture) -> None:
    spy = mocker.spy(datalad_installer, "runcmd")
    r = main(["datalad_installer.py", "git-annex", "-m", "brew"])
    assert r == 0
    assert spy.call_args_list[-1] == mocker.call("brew", "install", "git-annex")
    assert shutil.which("git-annex") is not None


@pytest.mark.ghauth
@pytest.mark.parametrize(
    "ostype,ext",
    [
        ("ubuntu", ".deb"),
        ("macos", ".dmg"),
        ("windows", ".exe"),
    ],
)
def test_download_latest_git_annex_release_asset(
    ostype: str, ext: str, tmp_path: Path
) -> None:
    DataladGitAnnexReleaseBuildInstaller.download(
        ostype=ostype,
        target_dir=tmp_path,
        version=None,
    )
    (p,) = tmp_path.iterdir()
    assert p.is_file()
    assert p.suffix == ext
    assert p.stat().st_size >= (1 << 20)  # 1 MiB


@pytest.mark.ghauth
@pytest.mark.parametrize(
    "ostype,version,filename,size",
    [
        (
            "ubuntu",
            "8.20211231",
            "git-annex-standalone_8.20211231-1.ndall+1_amd64.deb",
            49966540,
        ),
        ("macos", "8.20211123", "git-annex_8.20211123_x64.dmg", 28809446),
        ("windows", "8.20211117", "git-annex-installer_8.20211117_x64.exe", 18156853),
    ],
)
def test_download_specific_git_annex_release_asset(
    ostype: str, version: str, filename: str, size: int, tmp_path: Path
) -> None:
    DataladGitAnnexReleaseBuildInstaller.download(
        ostype=ostype,
        target_dir=tmp_path,
        version=version,
    )
    (p,) = tmp_path.iterdir()
    assert p.is_file()
    assert p.name == filename
    assert p.stat().st_size == size


@pytest.mark.skipif(not ON_POSIX, reason="POSIX only")
def test_install_git_annex_remote_rclone_latest_from_github(tmp_path: Path) -> None:
    r = main(
        [
            "datalad_installer.py",
            "git-annex-remote-rclone",
            "-m",
            "DanielDent/git-annex-remote-rclone",
            "--bin-dir",
            str(tmp_path / "bin"),
        ]
    )
    assert r == 0
    (p,) = (tmp_path / "bin").iterdir()
    assert p.is_file()
    assert p.name == "git-annex-remote-rclone"
    assert p.stat().st_size >= 5120


@pytest.mark.skipif(not ON_POSIX, reason="POSIX only")
def test_install_git_annex_remote_rclone_specific_version_from_github(
    tmp_path: Path,
) -> None:
    r = main(
        [
            "datalad_installer.py",
            "git-annex-remote-rclone=0.5",
            "-m",
            "DanielDent/git-annex-remote-rclone",
            "--bin-dir",
            str(tmp_path / "bin"),
        ]
    )
    assert r == 0
    (p,) = (tmp_path / "bin").iterdir()
    assert p.is_file()
    assert p.name == "git-annex-remote-rclone"
    assert p.stat().st_size == 7120


@pytest.mark.ci_only
@pytest.mark.needs_sudo
@pytest.mark.skipif(not ON_POSIX, reason="POSIX only")
def test_install_git_annex_remote_rclone_latest_from_github_globally() -> None:
    r = main(
        [
            "datalad_installer.py",
            "--sudo=ok",
            "git-annex-remote-rclone",
            "-m",
            "DanielDent/git-annex-remote-rclone",
        ]
    )
    assert r == 0
    p = Path("/usr/local/bin/git-annex-remote-rclone")
    assert p.is_file()
    assert p.stat().st_size >= 5120


def test_install_latest_rclone_from_downloads(tmp_path: Path) -> None:
    r = main(
        [
            "datalad_installer.py",
            "rclone",
            "-m",
            "downloads.rclone.org",
            "--bin-dir",
            str(tmp_path / "bin"),
        ]
    )
    assert r == 0
    (p,) = (tmp_path / "bin").iterdir()
    assert p.is_file()
    if ON_WINDOWS:
        assert p.name == "rclone.exe"
    else:
        assert p.name == "rclone"
    assert p.stat().st_size >= (1 << 20)  # 1 MiB


def test_install_latest_rclone_from_downloads_with_manpage(tmp_path: Path) -> None:
    r = main(
        [
            "datalad_installer.py",
            "rclone",
            "-m",
            "downloads.rclone.org",
            "--bin-dir",
            str(tmp_path / "bin"),
            "--man-dir",
            str(tmp_path / "man"),
        ]
    )
    assert r == 0
    (p,) = (tmp_path / "bin").iterdir()
    assert p.is_file()
    if ON_WINDOWS:
        assert p.name == "rclone.exe"
    else:
        assert p.name == "rclone"
    assert p.stat().st_size >= (1 << 20)  # 1 MiB
    (m,) = (tmp_path / "man" / "man1").iterdir()
    assert m.is_file()
    assert m.name == "rclone.1"
    assert m.stat().st_size >= (1 << 20)  # 1 MiB


@pytest.mark.parametrize(
    "version,size",
    [
        pytest.param(
            "1.55.0",
            43130880,
            marks=pytest.mark.skipif(not ON_LINUX, reason="Linux only"),
        ),
        pytest.param(
            "v1.54.1",
            58878352,
            marks=pytest.mark.skipif(not ON_MACOS, reason="macOS only"),
        ),
        pytest.param(
            "1.56.2",
            44712960,
            marks=pytest.mark.skipif(not ON_WINDOWS, reason="Windows only"),
        ),
    ],
)
def test_install_rclone_version_from_downloads(
    mocker: MockerFixture, tmp_path: Path, version: str, size: int
) -> None:
    mocker.patch("platform.machine", return_value="x86_64")
    r = main(
        [
            "datalad_installer.py",
            f"rclone={version}",
            "-m",
            "downloads.rclone.org",
            "--bin-dir",
            str(tmp_path / "bin"),
        ]
    )
    assert r == 0
    (p,) = (tmp_path / "bin").iterdir()
    assert p.is_file()
    if ON_WINDOWS:
        assert p.name == "rclone.exe"
    else:
        assert p.name == "rclone"
    assert p.stat().st_size == size


@pytest.mark.ci_only
@pytest.mark.needs_sudo
@pytest.mark.skipif(not ON_POSIX, reason="POSIX only")
def test_install_latest_rclone_from_downloads_globally(mocker: MockerFixture) -> None:
    spy = mocker.spy(datalad_installer, "runcmd")
    r = main(
        [
            "datalad_installer.py",
            "--sudo=ok",
            "rclone",
            "-m",
            "downloads.rclone.org",
        ]
    )
    assert r == 0
    p = Path("/usr/local/bin/rclone")
    if spy.called:
        spy.assert_called_once_with("sudo", "mv", "-f", "--", mocker.ANY, str(p))
    assert p.is_file()
    assert p.stat().st_size >= (1 << 20)  # 1 MiB



from typing import List
import pytest


def pytest_addoption(parser: pytest.Parser) -> None:
    parser.addoption(
        "--ci",
        action="store_true",
        default=False,
        help="Enable CI-only tests",
    )


def pytest_collection_modifyitems(
    config: pytest.Config, items: List[pytest.Item]
) -> None:
    if not config.getoption("--ci"):
        skip_no_ci = pytest.mark.skip(reason="Only run when --ci is given")
        for item in items:
            if "ci_only" in item.keywords:
                item.add_marker(skip_no_ci)
